use crate::{gui::{VRAM_HEIGHT, VRAM_WIDTH, create_vram_bg}, MainWindow, VRAMImage};

use slint::Model;
use std::{cell::RefCell, ffi::OsStr, path::PathBuf, rc::Rc};
use tool_helper::Error;

pub struct GUIElements {
    pub main_window:     Rc<RefCell<MainWindow>>,
        vram_file_list:  Rc<slint::VecModel<slint::StandardListViewItem>>,
        vram_image_list: Rc<slint::VecModel<VRAMImage>>,
}

impl GUIElements {
    pub fn new(main_window: Rc<RefCell<MainWindow>>) -> GUIElements {
        let vram_file_list:Vec<slint::StandardListViewItem> = main_window.borrow().get_main_tab_vram_file_list().iter().collect();
        let vram_file_list                                  = Rc::new(slint::VecModel::from(vram_file_list));

        let vram_image_list = Rc::new(slint::VecModel::from(Vec::<VRAMImage>::new()));

        main_window.borrow().set_main_tab_vram_file_list(vram_file_list.clone().into());
        main_window.borrow().set_main_tab_vram_images(vram_image_list.clone().into());

        GUIElements{main_window, vram_file_list, vram_image_list}
    }

    pub fn add_new_vram_file(&mut self, file: PathBuf) -> Result<(), Error> {
        let vram_image = VRAMImage{
            img: slint::Image::load_from_path(&file).or_else(|_| {Err(Error::from_str("Failed loading image"))})?,
            x:   0,
            y:   0
        };

        let img_size = vram_image.img.size();
        if img_size.width > VRAM_WIDTH as u32 || img_size.height > VRAM_HEIGHT as u32 {
            return Err(Error::from_text(format!("Image size ({}; {}) is to big for VRAM ({}, {})", img_size.width, img_size.height, VRAM_WIDTH, VRAM_HEIGHT)));
        }

        let file_name = file.file_name().get_or_insert(OsStr::new("<No file name>")).to_string_lossy().to_string();
        self.vram_file_list.push(slint::StandardListViewItem::from(file_name.as_str()));
        self.vram_image_list.push(vram_image);
        Ok(())
    }

    pub fn remove_vram_file(&mut self, idx: usize) {
        self.vram_file_list.remove(idx);
        self.vram_image_list.remove(idx);
    }

    pub fn move_vram_image(&mut self, idx: usize, dx: i32, dy: i32) {
        if let Some(mut vram_info) = self.vram_image_list.row_data(idx) {
            vram_info.x += dx;
            vram_info.y += dy;

            if vram_info.x < 0 {
                vram_info.x = 0;
            }

            if vram_info.y < 0 {
                vram_info.y = 0;
            }

            let (vram_img_width, vram_img_height) = (vram_info.img.size().width as i32, vram_info.img.size().height as i32);
            if (vram_info.x + vram_img_width) > VRAM_WIDTH as i32 {
                vram_info.x = VRAM_WIDTH as i32 - vram_img_width;
            }

            if (vram_info.y + vram_img_height) > VRAM_HEIGHT as i32 {
                vram_info.y = VRAM_HEIGHT as i32 - vram_img_height;
            }

            self.vram_image_list.set_row_data(idx, vram_info);
        }
    }

    pub fn create_bg() -> Result<slint::Image, slint::PlatformError> {
        Ok(slint::Image::from_rgba8_premultiplied(create_vram_bg(320, 256).ok_or(slint::PlatformError::Other("Failed creating VRAM background image".to_string()))?))
    }
}